<?php
// $Id: dynamic_formatters_renderer.inc,v 1.1.2.5 2011/01/24 07:56:29 dixon Exp $

/**
 * @file
 * This is the render class for a dynamic views style.
 *
 * @todo
 *   Find a better way to handler the group wrapping.
 */

class dynamic_formatters_renderer {
  /**
   * Region key.
   */
  public $region_key = 'default';

  /**
   * Variant key.
   */
  public $variant_key = NULL;

  /**
   * Various ptions.
   */
  public $options = array();

  /**
   * Holds all raw row data.
   */
  public $rows = array();

  /**
   * Keeps track on which group index we're at.
   */
  public $group_index = 0;

  /**
   * Keeps track on which row index we're at.
   */
  public $row_index = 0;

  /**
   * Keeps track on which index we're at, in a specific group keyed by the
   * parent.
   */
  public $child_index = array();

  /**
   * Keeps track on how many childs there are per parent key. Basically
   * the same as $this->child_index but doesn't get reset at any time.
   */
  public $child_count = array();

  /**
   * Holds a map between row indexes and row keys.
   */
  public $index_map = array();

  /**
   * Keeps track of the last row keyed by parent.
   */
  public $last = array();

  /**
   * Tells if all the rows are built or not.
   */
  public $built = FALSE;

  /**
   * Constructor.
   *
   * @todo
   *   Is it cleaner to pass in a complete style object here?
   */
  function __construct($region_key = NULL, $variant_key = NULL, $options = array()) {
    $this->region_key = $region_key;
    $this->variant_key = $variant_key;
    $this->options = $options;
  }

  /**
   * Builds all the rows. This function does the heavy lifting with finding out
   * which styles to inherit and use.
   *
   * @todo
   *   What happens if the key doesn't exist in the row?
   *
   * @todo
   *   What happens if the parent isn't present in the view?
   */
  public function build_row($key, $reset = FALSE) {
    if (!$this->rows[$key]['built'] || $reset) {
      // We have one global style object for the whole view.
      if (!empty($this->options['preset_name'])) {
        // Load the style object.
        $this->rows[$key]['style'] = $this->get_style($this->options['preset_name']);
        // Move the index forward.
        $this->move_row_index();
      }
      // Each row has it's own style object. But this row has a parent that it
      // will inherit the style object from.
      elseif (!empty($this->rows[$key]['parent'])) {
        $parent = $this->build_row($this->rows[$key]['parent']);
        $parent_key = $this->rows[$key]['parent'];
        // This row inherits the style object from it's parent.
        $this->rows[$key]['preset_name'] = $parent['preset_name'];
        // Give this row some formatters.
        $this->rows[$key]['style'] = $this->get_style($parent['preset_name'], NULL, $parent_key);
        // Save this as the last children for this parent. Later children will
        // override this.
        $this->last[$parent_key] = $key;
        // Move the index forward.
        $this->move_row_index($parent_key);
        $this->increase_child_count($parent_key);
      }
      // Each row has it's own style object. This row doesn't have a parent, so
      // it must be first in it's own group.
      else {
        // Load it's style object.
        $this->rows[$key]['style'] = $this->get_style($this->rows[$key]['preset_name'], 'first');
        // The last children is the parent it self, to start with.
        $this->last[$key] = $key;
        // Move the index forward.
        $this->move_group_index();
        $this->move_row_index($key);
        $this->increase_child_count($key);
      }
      // Mark this row as built.
      $this->rows[$key]['built'] = TRUE;
    }
    return $this->rows[$key];
  }

  /**
   * Processes all rows.
   *
   * @todo
   *   Is it worth having a separate loop for this? Maybe this shall go
   *   directly into $this->render()?
   *
   * @todo
   *   Is it ok to have a static cache here? Or should we implement a
   *   class member for this?
   */
  public function build_rows($reset = FALSE) {
    if (!$this->built || $reset) {
      $this->reset_group_index();
      $this->reset_row_index();

      foreach ($this->rows as $key => $row) {
        // The index is moved inside this function.
        $this->build_row($key);
      }
      $this->built = TRUE;
    }
  }

  /**
   * Returns the style to use.
   *
   * @todo
   *   Make this function able to inherit all keys and properities from
   *   'default' if they are not defined as empty.
   */
  public function get_style($preset_name, $style_name = NULL, $parent_key = NULL) {
    $style = array();

    // If no style name, find it.
    if (empty($style_name)) {
      if ($this->is_first($parent_key)) {
        $style_name = 'first';
      }
      elseif ($this->is_odd($parent_key)) {
        $style_name = 'odd';
      }
      else {
        $style_name = 'even';
      }
    }

    // Get the style.
    $preset = dynamic_formatters_preset_load($preset_name);

    // If no preset, return empty mock style.
    if (empty($preset)) {
      return array(
        'prefix' => '',
        'suffix' => '',
        'formatters' => array(),
        'group' => array(
          'prefix' => '',
          'suffix' => '',
        ),
      );
    }

    $style = $preset->get_style($this->region_key, $this->variant_key, $style_name);

    // The last thing we do is to make the substitutions for @counter in
    // 'prefix' and 'suffix'.
    $this->substitute_counter($style, $parent_key);

    return $style;
  }

  /**
   * Function that replaces the %counter placeholder.
   *
   * @todo
   *   Move the substitution of %count here too.
   */
  public function substitute_counter(&$style, $parent_key) {
    $row_index = $this->get_row_index($parent_key);
    $group_index = $this->get_group_index();

    $style['prefix'] = strtr($style['prefix'], array('%counter' => $row_index));
    $style['suffix'] = strtr($style['suffix'], array('%counter' => $row_index));
    $style['group']['prefix'] = strtr($style['group']['prefix'], array('%counter' => $group_index));
    $style['group']['suffix'] = strtr($style['group']['suffix'], array('%counter' => $group_index));
  }

  /**
   * Returns a row.
   */
  public function get_row($key = NULL) {
    if (empty($key)) {
      $key = $this->index_map[$this->get_row_index()];
    }
    return $this->rows[$key];
  }

  /**
   * Sets the properties for one row.
   *
   * @todo
   *   Is it cleaner to pass in a complete style object here?
   *
   * @todo
   *   Maybe we can count the children here. We have access to the parent key.
   *   Then we could do the substitution together with $this->substitute_counter().
   */
  public function prepare_row($key, $preset_name = NULL, $parent = NULL) {
    $this->rows[$key] = array(
      'key' => $key,
      'preset_name' => $preset_name,
      'parent' => $parent,
    );
    $this->index_map[] = $key;
  }

  /**
   * Render the output for one row.
   *
   * @todo
   *   Move the count substitution to a more logical place.
   */
  public function render_row($content = NULL) {
    $row = $this->get_row();
    $group_prefix = '';
    $group_suffix = '';

    // If we doesn't have a global style object, and we have an active parent
    // field, that means that all parent shall open a new group.
    if (empty($this->options['preset_name']) && !empty($this->options['parent_field'])) {
      $group_key = $row['parent'];
      if (empty($row['parent'])) {
        $group_key = $row['key'];
        $group_prefix = strtr($row['style']['group']['prefix'], array('%count' => $this->get_child_count($group_key)));
      }
      if ($this->last[$group_key] == $row['key']) {
        $group_suffix = strtr($row['style']['group']['suffix'], array('%count' => $this->get_child_count($group_key)));
      }
    }

    $output = $group_prefix . $row['style']['prefix'] . $content . $row['style']['suffix'] . $group_suffix;

    if (!empty($group_suffix)) {
      $modules = module_implements('dynamic_formatters_group_alter');
      foreach ($modules as $module) {
        $function = $module . '_dynamic_formatters_group_alter';
        $function($this->region_key, $output);
      }
    }
    return $output;
  }

  /**
   * Render the output of all rows.
   */
  public function render_rows($rows = array()) {
    // From now on we will only render rows with the global index in mind.
    // So we must reset that index to be able to render all rows.
    $this->reset_group_index();
    $this->reset_row_index();

    $output = '';
    foreach ($rows as $content) {
      $output .= $this->render_row($content);
      $this->move_row_index();
    }
    return $output;
  }

  /**
   * If a key is provided this returns TRUE if the keyed row is first in it's
   * parent's group or not. If a key is not provided this returns TRUE if the
   * row is first in the whole view.
   */
  public function is_first($parent_key = NULL) {
    return $this->get_row_index($parent_key) == 0;
  }

  /**
   * If a key is provided this returns TRUE if the keyed row is of an odd index
   * in it's parent's group or not. If a key is not provided this returns TRUE
   * if the row of an odd index in the whole view.
   */
  public function is_odd($parent_key = NULL) {
    return !($this->get_row_index($parent_key) % 2);
  }

  /**
   * Returns the group index.
   */
  public function get_group_index() {
    return $this->group_index;
  }

  /**
   * Returns the current index.
   */
  public function get_row_index($key = NULL) {
    if (empty($key)) {
      return $this->row_index;
    }
    return isset($this->child_index[$key]) ? $this->child_index[$key] : 0;
  }

  /**
   * Moves the index one step forward for groups.
   */
  public function move_group_index() {
    $this->group_index++;
  }

  /**
   * Moves the index one step forward for rows.
   */
  public function move_row_index($parent_key = NULL) {
    if (empty($parent_key)) {
      $this->row_index++;
    }
    else {
      $this->child_index[$parent_key]++;
    }
  }

  /**
   * Increase the child count for a specified parent key.
   */
  public function increase_child_count($parent_key) {
    if (!isset($this->child_count[$parent_key])) {
      $this->child_count[$parent_key] = 0;
    }
    $this->child_count[$parent_key]++;
  }

  /**
   * Return the child count for a given parent key.
   */
  public function get_child_count($parent_key) {
    return $this->child_count[$parent_key];
  }

  /**
   * Reset group index.
   */
  public function reset_group_index() {
    $this->group_index = 0;
  }

  /**
   * Resets the row index.
   */
  public function reset_row_index($parent_key = NULL) {
    if (empty($parent_key)) {
      $this->row_index = 0;
    }
    else {
      $this->child_index[$parent_key] = 0;
    }
  }
}
