<?php
// $Id: dynamic_formatters_ui.class.php,v 1.1.2.10 2011/01/22 22:47:13 dixon Exp $

/**
 * @file
 * Implementation of CTools Export UI.
 *
 * @todo Document UI.
 */

class dynamic_formatters_ui extends ctools_export_ui {

  function list_sort_options() {
    return array(
      'disabled' => t('Enabled, title'),
      'title' => t('Title'),
      'name' => t('Name'),
      'storage' => t('Storage'),
    );
  }

  function list_build_row($item, &$form_state, $operations) {
    // Set up sorting
    switch ($form_state['values']['order']) {
      case 'disabled':
        $this->sorts[$item->name] = empty($item->disabled) . $item->title;
        break;
      case 'title':
        $this->sorts[$item->name] = $item->title;
        break;
      case 'name':
        $this->sorts[$item->name] = $item->name;
        break;
      case 'storage':
        $this->sorts[$item->name] = $item->type . $item->title;
        break;
    }

    $this->rows[$item->name] = array(
      'data' => array(
        array('data' => check_plain($item->title), 'class' => 'ctools-export-ui-title'),
        array('data' => check_plain($item->name), 'class' => 'ctools-export-ui-name'),
        array('data' => check_plain($item->description), 'class' => 'ctools-export-ui-description'),
        array('data' => $item->type, 'class' => 'ctools-export-ui-storage'),
        array('data' => theme('links', $operations), 'class' => 'ctools-export-ui-operations'),
      ),
      'class' => !empty($item->disabled) ? 'ctools-export-ui-disabled' : 'ctools-export-ui-enabled',
    );
  }

  function list_table_header() {
    return array(
      array('data' => t('Title'), 'class' => 'ctools-export-ui-title'),
      array('data' => t('Name'), 'class' => 'ctools-export-ui-name'),
      array('data' => t('Description'), 'class' => 'ctools-export-ui-description'),
      array('data' => t('Storage'), 'class' => 'ctools-export-ui-storage'),
      array('data' => t('Operations'), 'class' => 'ctools-export-ui-operations'),
    );
  }

  /**
   * Form callback for basic settings.
   */
  function edit_form(&$form, &$form_state) {
    $item = $form_state['item'];
    $form['name'] = array(
      '#type' => 'textfield',
      '#title' => t('Machine-readable name'),
      '#default_value' => $item->name,
      '#required' => TRUE,
    );
    $form['title'] = array(
      '#type' => 'textfield',
      '#title' => t('Title'),
      '#default_value' => $item->title,
      '#required' => TRUE,
    );
    $form['description'] = array(
      '#type' => 'textarea',
      '#title' => t('Description'),
      '#default_value' => $item->description,
    );
  }

  /**
   * Submit callback for basic settings.
   */
  function edit_form_submit(&$form, &$form_state) {
    $form_state['item']->name = $form_state['values']['name'];
    $form_state['item']->title = $form_state['values']['title'];
    $form_state['item']->description = $form_state['values']['description'];
  }

  /**
   * Form callback for regions.
   */
  function edit_form_region(&$form, &$form_state) {
    $preset_name = check_plain($form_state['item']->name);
    $form['regions']['#tree'] = TRUE;
    $form['regions']['#prefix'] = '<div id="regions">';
    $form['regions']['#suffix'] = '</div>';
    if (module_exists('vertical_tabs')) {
      //$form['regions']['#pre_render'][] = 'vertical_tabs_form_pre_render';
    }

    // Make sure we always have a default region.
    if (empty($form_state['item']->regions)) {
      $form_state['item']->regions = array(
        'default' => array(
          'key' => 'default',
          'title' => t('Default'),
          'variants' => array(),
        ),
      );
    }

    foreach ($form_state['item']->regions as $region) {
      // Settings for the "default" region is required.
      $required = ($region['key'] == 'default') ? TRUE : FALSE;
      $form['regions'][$region['key']] = dynamic_formatter_region_form($preset_name, $region);
    }

    $form['add'] = array(
      '#type' => 'submit',
      '#value' => t('Add region'),
      '#prefix' => '<div class="clear-block">',
      '#suffix' => '</div>',
      '#ahah' => array(
        'path' => 'admin/dynamic-formatters/ahah/' . $preset_name . '/region/add',
        'wrapper' => 'regions',
        'method' => 'append',
        'effect' => 'fade',
      ),
    );
  }

  /**
   * Submit callback for regions.
   */
  function edit_form_region_submit(&$form, &$form_state) {
    foreach ($form_state['values']['regions'] as $region_key => $region_values) {
      // Save old variants.
      $region_variants = isset($form_state['item']->regions[$region_values['key_old']]['variants']) ? $form_state['item']->regions[$region_values['key_old']]['variants'] : array();
      // Remove old region.
      unset($form_state['item']->regions[$region_values['key_old']]);
      // Create new region with new values, but old variants.
      if ($region_values['key']) {
        $form_state['item']->regions[$region_values['key']]['key'] = $region_values['key'];
        $form_state['item']->regions[$region_values['key']]['title'] = $region_values['title'];
        $form_state['item']->regions[$region_values['key']]['variants'] = $region_variants;
      }
    }
  }

  /**
   * Form callback for variants.
   */
  function edit_form_variant(&$form, &$form_state) {
    $preset_name = check_plain($form_state['item']->name);
    $form['regions']['#tree'] = TRUE;

    foreach ($form_state['item']->regions as $region) {
      // Sanitize data once before used in form.
      $region_key = check_plain($region['key']);
      $region_title = check_plain($region['title']);

      $form['regions'][$region['key']]['heading']['#value'] = '<h2>' . $region_title . '</h2>';
      $form['regions'][$region['key']]['variants']['#prefix'] = '<div id="region-' . $region_key . '">';
      $form['regions'][$region['key']]['variants']['#suffix'] = '</div>';
      if (module_exists('vertical_tabs')) {
        //$form['regions'][$region['key']]['variants']['#pre_render'][] = 'vertical_tabs_form_pre_render';
      }

      // Make sure we always have a default variant.
      if (empty($region['variants'])) {
        $region['variants'] = array(
          'default' => array(
            'key' => 'default',
            'title' => t('Default'),
            'styles' => array(),
          ),
        );
      }

      foreach ($region['variants'] as $variant) {
        if (empty($variant['key'])) {
          continue;
        }
        $form['regions'][$region['key']]['variants'][$variant['key']] = dynamic_formatter_variant_form($preset_name, $region_key, $variant);
      }

      $form['regions'][$region['key']]['add'] = array(
        '#type' => 'submit',
        '#value' => t('Add variant'),
        '#prefix' => '<div class="clear-block">',
        '#suffix' => '</div>',
        '#ahah' => array(
          'path' => 'admin/dynamic-formatters/ahah/' . $preset_name . '/variant/add/' . $region_key,
          'wrapper' => 'region-' . $region_key,
          'method' => 'append',
          'effect' => 'fade',
        ),
      );
    }
  }

  /**
   * Submit callback for variants.
   */
  function edit_form_variant_submit(&$form, &$form_state) {
    foreach ($form_state['values']['regions'] as $region_key => $region_values) {
      foreach ($form_state['values']['regions'][$region_key]['variants'] as $variant_key => $variant_values) {
        // Save old styles.
        $variant_styles = isset($form_state['item']->regions[$region_key]['variants'][$variant_values['key_old']]['styles']) ? $form_state['item']->regions[$region_key]['variants'][$variant_values['key_old']]['styles'] : array();
        // Remove old variant.
        unset($form_state['item']->regions[$region_key]['variants'][$variant_values['key_old']]);
        // Create new variants with new values, but old styles.
        if ($variant_values['key_old']) {
          $form_state['item']->regions[$region_key]['variants'][$variant_values['key']]['key'] = $variant_values['key'];
          $form_state['item']->regions[$region_key]['variants'][$variant_values['key']]['title'] = $variant_values['title'];
          $form_state['item']->regions[$region_key]['variants'][$variant_values['key']]['styles'] = $variant_styles;
        }
      }
    }
  }

  /**
   * Submit callback for variants.
   */
  function edit_form_style(&$form, &$form_state) {
    $preset_name = check_plain($form_state['item']->name);
    $form['regions']['#tree'] = TRUE;

    foreach ($form_state['item']->regions as $region) {
      // Sanitize data once before used in form.
      $region_title = check_plain($region['title']);

      foreach ($region['variants'] as $variant) {
        // Sanitize data once before used in form.
        $variant_title = check_plain($variant['title']);

        $form['regions'][$region['key']]['variants'][$variant['key']]['heading']['#value'] = '<h2>' . $region_title . ': ' . $variant_title . '</h2>';
        if (module_exists('vertical_tabs')) {
          //$form['regions'][$region['key']]['variants'][$variant['key']]['styles']['#pre_render'][] = 'vertical_tabs_form_pre_render';
        }

        $styles = dynamic_formatter_get_styles();
        foreach ($styles as $style_name => $style_title) {
          $form['regions'][$region['key']]['variants'][$variant['key']]['styles'][$style_name] = dynamic_formatter_style_form($preset_name, $region_key, $variant, $style_name, $style_title);
        }
      }
    }
  }

  /**
   * Submit callback for styles.
   */
  function edit_form_style_submit(&$form, &$form_state) {
    foreach ($form_state['values']['regions'] as $region_key => $region_values) {
      foreach ($form_state['values']['regions'][$region_key]['variants'] as $variant_key => $variant_values) {
        foreach ($form_state['values']['regions'][$region_key]['variants'][$variant_key]['styles'] as $style_name => $style_values) {
          // Create.
          if (!isset($form_state['item']->regions[$region_key]['variants'][$variant_key]['styles'][$style_name])) {
            $form_state['item']->regions[$region_key]['variants'][$variant_key]['styles'][$style_name] = $style_values;
          }
          // Update.
          else {
            $form_state['item']->regions[$region_key]['variants'][$variant_key]['styles'][$style_name] = array_merge($form_state['item']->regions[$region_key]['variants'][$variant_key]['styles'][$style_name], $style_values);
          }
        }
      }
    }
  }
}
