<?php
// $Id: template.php,v 1.1.2.2 2011/02/14 12:26:28 fabsor Exp $

/**
 * Preprocessing the node template.
 *
 * This is basically here to unify and clean up some class names.
 */
function ns_theme_preprocess_node(&$vars) {
  $node = $vars['node'];
  $classes = array();
  $classes[] = 'node';
  $classes[] = 'node-' . strtr(check_plain($node->type), '_', '-');

  if ($node->sticky) {
    $classes[] = 'node-sticky';
  }
  if (empty($vars['classes'])) {
    $vars['classes'] = implode(' ', $classes);
  }
  else {
    $vars['classes'] = implode(' ', $classes) . ' ' . $vars['classes'];
  }
}

/**
 * Preprocessing the panel template.
 *
 * This is basically here to unify and clean up some class names.
 */
function ns_theme_preprocess_panels_pane(&$vars) {
  // Remove the old class.
  $vars['classes'] = strtr($vars['classes'], array('_' => '-'));

  // Adds the total number of comments to a node right next to the comment form title
  if ($vars['pane']->type == 'node_comment_form') {
    $comment_count = comment_num_all($vars['output']->delta);
    if ($comment_count > 0) {
      $title = '<span>' . $vars['output']->title . '</span><span class="total-comment-count">(';
      $title .= t('Total !comments', array(
          '!comments' => format_plural($comment_count, '1 comment', '@count comments')));
      $title .= ")</span>";
      $vars['title'] = $title;
    }
  }
}

/**
 * Theme function for the page status messages. This adds a placeholder
 * for a close icon. A custom javascript is added to the page for the behavior.
 */
function ns_theme_status_messages($display = NULL) {
  $output = '';
  foreach (drupal_get_messages($display) as $type => $messages) {
    $output .= "<div class=\"messages $type\"><span class=\"icon\"></span>\n";
    if (count($messages) > 1) {
      $output .= " <ul>\n";
      foreach ($messages as $message) {
        $output .= '  <li>'. $message ."</li>\n";
      }
      $output .= " </ul>\n";
    }
    else {
      $output .= $messages[0];
    }
    $output .= "</div>\n";
  }
  return $output;
}


function ns_theme_semanticviews_view_fields__ns_blog($view, $options, $row) {
  $vars = array('view' => $view, 'options' => $options);
  template_preprocess_semanticviews_view_fields($vars);
  $fields = $vars['fields'];
  $image_field = $fields['field_ns_blog_image_fid'];
  unset($fields['field_ns_blog_image_fid']);
  $output = '<div class="blog-image">';
  $output .= _ns_theme_ns_blog_field($view, $row, $image_field);
  $output .= '</div><div class="blogs-overview-wrapper">';
  foreach ($fields as $field) {
    $output .= _ns_theme_ns_blog_field($view, $fields, $field);
  }
  $output .= '</div>';
  return $output;
}

function _ns_theme_ns_blog_field($view, $fields, $field) {

    if ($field->element_type) :
      $output .= '<' . $field->element_type;
      $output .= drupal_attributes($field->attributes) . '>';
    endif;

      if ($field->label):

        if ($field->label_element_type):
          $output .= '<' . $field->label_element_type;
          $output .= drupal_attributes($field->label_attributes) . '>';
        endif;

            $output .= $field->label;

        if ($field->label_element_type):
          $output .= '</' . $field->label_element_type . '>';
        endif;

      endif;

        $output .= $field->content;

    if ($field->element_type):
      $output .= '</' . $field->element_type . '>';
    endif;
    
  return $output;

}

/*
 * theme_comment_submitted
 */
function ns_theme_comment_submitted($comment) {
  return t('!username, @datetime',
    array(
    '!username' => theme('username', $comment),
    '@datetime' => format_date($comment->timestamp, 'custom', 'H:i F j, Y'),
  ));
}
